@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -65,7 +65,6 @@ DocumentationWidget::DocumentationWidget(QWidget *pParent)
   : QWidget(pParent)
 {
   setObjectName("DocumentationWidget");
-  setMinimumWidth(175);
   mDocumentationFile.setFileName(Utilities::tempDirectory() + "/DocumentationWidget.html");
   // documentation toolbar
   QToolBar *pDocumentationToolBar = new QToolBar;
@@ -118,6 +117,13 @@ DocumentationWidget::DocumentationWidget(QWidget *pParent)
   pDocumentationToolBar->addAction(mpCancelAction);
   // create the documentation viewer
   mpDocumentationViewer = new DocumentationViewer(this, false);
+  mpDocumentationViewerFrame = new QFrame;
+  mpDocumentationViewerFrame->setContentsMargins(0, 0, 0, 0);
+  mpDocumentationViewerFrame->setFrameStyle(QFrame::StyledPanel);
+  QHBoxLayout *pDocumentationViewerLayout = new QHBoxLayout;
+  pDocumentationViewerLayout->setContentsMargins(0, 0, 0, 0);
+  pDocumentationViewerLayout->addWidget(mpDocumentationViewer);
+  mpDocumentationViewerFrame->setLayout(pDocumentationViewerLayout);
   // create the editors tab widget
   mpEditorsWidget = new QWidget;
   mpEditorsWidget->hide();
@@ -134,6 +140,13 @@ DocumentationWidget::DocumentationWidget(QWidget *pParent)
   mpEditorToolBar->setIconSize(QSize(toolbarIconSize, toolbarIconSize));
   // create the html editor viewer
   mpHTMLEditor = new DocumentationViewer(this, true);
+  mpHTMLEditorFrame = new QFrame;
+  mpHTMLEditorFrame->setContentsMargins(0, 0, 0, 0);
+  mpHTMLEditorFrame->setFrameStyle(QFrame::StyledPanel);
+  QHBoxLayout *pHTMLEditorLayout = new QHBoxLayout;
+  pHTMLEditorLayout->setContentsMargins(0, 0, 0, 0);
+  pHTMLEditorLayout->addWidget(mpHTMLEditor);
+  mpHTMLEditorFrame->setLayout(pHTMLEditorLayout);
   // editor actions
   // style combobox
   mpStyleComboBox = new QComboBox;
@@ -345,7 +358,7 @@ DocumentationWidget::DocumentationWidget(QWidget *pParent)
   pHTMLWidgetLayout->setContentsMargins(0, 0, 0, 0);
   pHTMLWidgetLayout->setSpacing(0);
   pHTMLWidgetLayout->addWidget(mpEditorToolBar);
-  pHTMLWidgetLayout->addWidget(mpHTMLEditor, 1);
+  pHTMLWidgetLayout->addWidget(mpHTMLEditorFrame, 1);
   mpHTMLEditorWidget->setLayout(pHTMLWidgetLayout);
   // create the HTMLEditor
   mpHTMLSourceEditor = new HTMLEditor(this);
@@ -369,7 +382,7 @@ DocumentationWidget::DocumentationWidget(QWidget *pParent)
   // Documentation viewer layout
   QGridLayout *pGridLayout = new QGridLayout;
   pGridLayout->setContentsMargins(0, 0, 0, 0);
-  pGridLayout->addWidget(mpDocumentationViewer);
+  pGridLayout->addWidget(mpDocumentationViewerFrame);
   pGridLayout->addWidget(mpEditorsWidget);
   QVBoxLayout *pMainLayout = new QVBoxLayout;
   pMainLayout->setContentsMargins(0, 0, 0, 0);
@@ -395,6 +408,16 @@ DocumentationWidget::~DocumentationWidget()
  */
 void DocumentationWidget::showDocumentation(LibraryTreeItem *pLibraryTreeItem)
 {
+  // We only support documentation of Modelica classes.
+  if (pLibraryTreeItem->getLibraryType() != LibraryTreeItem::Modelica) {
+    return;
+  }
+  // if documentation is proctected then do not show it.
+  if (pLibraryTreeItem->getAccess() < LibraryTreeItem::documentation) {
+    // Remove the class documentation if it is showed previously.
+    updateDocumentationHistory(pLibraryTreeItem);
+    return;
+  }
   if (mEditType != EditType::None) {
     saveDocumentation(pLibraryTreeItem);
     return;
@@ -423,7 +446,7 @@ void DocumentationWidget::showDocumentation(LibraryTreeItem *pLibraryTreeItem)
   mpEditInfoHeaderAction->setDisabled(pLibraryTreeItem->isSystemLibrary());
   mpSaveAction->setDisabled(true);
   mpCancelAction->setDisabled(true);
-  mpDocumentationViewer->show();
+  mpDocumentationViewerFrame->show();
   mpEditorsWidget->hide();
 }
 
@@ -535,7 +558,7 @@ void DocumentationWidget::writeDocumentationFile(QString documentation)
   /* Create a local file with the html we want to view as otherwise JavaScript does not run properly. */
   mDocumentationFile.open(QIODevice::WriteOnly);
   QTextStream out(&mDocumentationFile);
-  out.setCodec(Helper::utf8.toStdString().data());
+  out.setCodec(Helper::utf8.toUtf8().constData());
   out << documentation;
   mDocumentationFile.close();
 }
@@ -560,6 +583,46 @@ bool DocumentationWidget::isLinkSelected()
   return result.toString().simplified().toLower() == "true";
 }
 
+/*!
+ * \brief DocumentationWidget::updateDocumentationHistory
+ * \param pLibraryTreeItem
+ * Removes the corresponding LibraryTreeItem from the DocumentationHistory.
+ */
+void DocumentationWidget::updateDocumentationHistory(LibraryTreeItem *pLibraryTreeItem)
+{
+  if (pLibraryTreeItem) {
+    int index = mpDocumentationHistoryList->indexOf(DocumentationHistory(pLibraryTreeItem));
+    if (index > -1) {
+      mpDocumentationHistoryList->removeOne(DocumentationHistory(pLibraryTreeItem));
+      if (index == mDocumentationHistoryPos) {
+        if (!(index == 0 && !mpDocumentationHistoryList->isEmpty())) {
+          mDocumentationHistoryPos--;
+        }
+      } else if (index < mDocumentationHistoryPos) {
+        mDocumentationHistoryPos--;
+      } else if (index > mDocumentationHistoryPos) {
+        // do nothing
+      }
+      updatePreviousNextButtons();
+      if (mDocumentationHistoryPos > -1) {
+        cancelDocumentation();
+        LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
+        pLibraryTreeModel->showModelWidget(mpDocumentationHistoryList->at(mDocumentationHistoryPos).mpLibraryTreeItem);
+      } else {
+        mpEditInfoAction->setDisabled(true);
+        mpEditRevisionsAction->setDisabled(true);
+        mpEditInfoHeaderAction->setDisabled(true);
+        mpSaveAction->setDisabled(true);
+        mpCancelAction->setDisabled(true);
+        mpDocumentationViewer->setHtml(""); // clear if we don't have any documentation to show
+        mpDocumentationViewerFrame->show();
+        mpEditorsWidget->hide();
+        mEditType = EditType::None;
+      }
+    }
+  }
+}
+
 /*!
  * \brief DocumentationWidget::previousDocumentation
  * Moves to the previous documentation.\n
@@ -569,7 +632,8 @@ void DocumentationWidget::previousDocumentation()
 {
   if (mDocumentationHistoryPos > 0) {
     mDocumentationHistoryPos--;
-    showDocumentation(mpDocumentationHistoryList->at(mDocumentationHistoryPos).mpLibraryTreeItem);
+    LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
+    pLibraryTreeModel->showModelWidget(mpDocumentationHistoryList->at(mDocumentationHistoryPos).mpLibraryTreeItem);
   }
 }
 
@@ -582,7 +646,8 @@ void DocumentationWidget::nextDocumentation()
 {
   if ((mDocumentationHistoryPos + 1) < mpDocumentationHistoryList->count()) {
     mDocumentationHistoryPos++;
-    showDocumentation(mpDocumentationHistoryList->at(mDocumentationHistoryPos).mpLibraryTreeItem);
+    LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
+    pLibraryTreeModel->showModelWidget(mpDocumentationHistoryList->at(mDocumentationHistoryPos).mpLibraryTreeItem);
   }
 }
 
@@ -613,7 +678,7 @@ void DocumentationWidget::editInfoDocumentation()
       mpEditInfoHeaderAction->setDisabled(true);
       mpSaveAction->setDisabled(false);
       mpCancelAction->setDisabled(false);
-      mpDocumentationViewer->hide();
+      mpDocumentationViewerFrame->hide();
       mpEditorsWidget->show();
       mpTabBar->setCurrentIndex(0);
       mpHTMLEditor->setFocusInternal();
@@ -648,7 +713,7 @@ void DocumentationWidget::editRevisionsDocumentation()
       mpEditInfoHeaderAction->setDisabled(true);
       mpSaveAction->setDisabled(false);
       mpCancelAction->setDisabled(false);
-      mpDocumentationViewer->hide();
+      mpDocumentationViewerFrame->hide();
       mpEditorsWidget->show();
       mpTabBar->setCurrentIndex(0);
       mpHTMLEditor->setFocusInternal();
@@ -683,7 +748,7 @@ void DocumentationWidget::editInfoHeaderDocumentation()
       mpEditInfoHeaderAction->setDisabled(true);
       mpSaveAction->setDisabled(false);
       mpCancelAction->setDisabled(false);
-      mpDocumentationViewer->hide();
+      mpDocumentationViewerFrame->hide();
       mpEditorsWidget->show();
       mpTabBar->setCurrentIndex(0);
       mpHTMLEditor->setFocusInternal();
@@ -763,6 +828,8 @@ void DocumentationWidget::saveDocumentation(LibraryTreeItem *pNextLibraryTreeIte
         LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
         pLibraryTreeModel->updateLibraryTreeItemClassText(pLibraryTreeItem);
       }
+      /* ticket:5190 Save the class when documentation save button is hit. */
+      MainWindow::instance()->getLibraryWidget()->saveLibraryTreeItem(pLibraryTreeItem);
       mEditType = EditType::None;
       showDocumentation(pNextLibraryTreeItem ? pNextLibraryTreeItem : pLibraryTreeItem);
     }
@@ -782,7 +849,7 @@ void DocumentationWidget::cancelDocumentation()
   mpEditInfoHeaderAction->setDisabled(false);
   mpSaveAction->setDisabled(true);
   mpCancelAction->setDisabled(true);
-  mpDocumentationViewer->show();
+  mpDocumentationViewerFrame->show();
   mpEditorsWidget->hide();
   mEditType = EditType::None;
 }
@@ -1039,42 +1106,11 @@ void DocumentationWidget::updateHTMLSourceEditor()
 
 /*!
  * \brief DocumentationWidget::updateDocumentationHistory
- * Slot activated when LibraryTreeItem unloaded SIGNAL is raised.\n
- * Removes the corresponding LibraryTreeItem from the DocumentationHistory.
+ * Slot activated when LibraryTreeItem unloaded SIGNAL is raised.
  */
 void DocumentationWidget::updateDocumentationHistory()
 {
-  LibraryTreeItem *pLibraryTreeItem = qobject_cast<LibraryTreeItem*>(sender());
-  if (pLibraryTreeItem) {
-    int index = mpDocumentationHistoryList->indexOf(DocumentationHistory(pLibraryTreeItem));
-    if (index > -1) {
-      mpDocumentationHistoryList->removeOne(DocumentationHistory(pLibraryTreeItem));
-      if (index == mDocumentationHistoryPos) {
-        if (!(index == 0 && !mpDocumentationHistoryList->isEmpty())) {
-          mDocumentationHistoryPos--;
-        }
-      } else if (index < mDocumentationHistoryPos) {
-        mDocumentationHistoryPos--;
-      } else if (index > mDocumentationHistoryPos) {
-        // do nothing
-      }
-      updatePreviousNextButtons();
-      if (mDocumentationHistoryPos > -1) {
-        cancelDocumentation();
-        showDocumentation(mpDocumentationHistoryList->at(mDocumentationHistoryPos).mpLibraryTreeItem);
-      } else {
-        mpEditInfoAction->setDisabled(true);
-        mpEditRevisionsAction->setDisabled(true);
-        mpEditInfoHeaderAction->setDisabled(true);
-        mpSaveAction->setDisabled(true);
-        mpCancelAction->setDisabled(true);
-        mpDocumentationViewer->setHtml(""); // clear if we don't have any documentation to show
-        mpDocumentationViewer->show();
-        mpEditorsWidget->hide();
-        mEditType = EditType::None;
-      }
-    }
-  }
+  updateDocumentationHistory(qobject_cast<LibraryTreeItem*>(sender()));
 }
 
 /*!
@@ -1097,7 +1133,7 @@ DocumentationViewer::DocumentationViewer(DocumentationWidget *pDocumentationWidg
   settings()->setFontFamily(QWebSettings::StandardFont, Helper::systemFontInfo.family());
   settings()->setFontSize(QWebSettings::DefaultFontSize, Helper::systemFontInfo.pointSize());
   settings()->setAttribute(QWebSettings::LocalStorageEnabled, true);
-  settings()->setDefaultTextEncoding(Helper::utf8.toStdString().data());
+  settings()->setDefaultTextEncoding(Helper::utf8.toUtf8().constData());
   // set DocumentationViewer web page policy
   page()->setContentEditable(isContentEditable);
   page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
@@ -1168,7 +1204,7 @@ void DocumentationViewer::processLinkClick(QUrl url)
       LibraryTreeItem *pLibraryTreeItem = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->findLibraryTreeItem(resourceLink);
       // send the new className to DocumentationWidget
       if (pLibraryTreeItem) {
-        mpDocumentationWidget->showDocumentation(pLibraryTreeItem);
+        MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->showModelWidget(pLibraryTreeItem);
       }
     }
   } else { // if it is normal http request then check if its not redirected to https
@@ -1231,17 +1267,6 @@ void DocumentationViewer::showContextMenu(QPoint point)
   menu.exec(mapToGlobal(point));
 }
 
-void DocumentationViewer::paintEvent(QPaintEvent *event)
-{
-  QWebView::paintEvent(event);
-  QPainter painter(this);
-  painter.setPen(Qt::gray);
-  QRect rectangle = rect();
-  rectangle.setWidth(rect().width() - 1);
-  rectangle.setHeight(height() - 1);
-  painter.drawRect(rectangle);
-}
-
 /*!
  * \brief DocumentationViewer::createWindow
  * \param type
