@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -42,10 +42,12 @@
 #include "Editors/BaseEditor.h"
 #include "Editors/ModelicaEditor.h"
 #include "Editors/CompositeModelEditor.h"
+#include "Editors/OMSimulatorEditor.h"
 #include "Editors/CEditor.h"
 #include "Editors/TextEditor.h"
 #include "Editors/MetaModelicaEditor.h"
 #include "LibraryTreeWidget.h"
+#include "OMSimulator.h"
 
 #include <QGraphicsView>
 #include <QGraphicsScene>
@@ -83,6 +85,7 @@ class GraphicsView : public QGraphicsView
 private:
   StringHandler::ViewType mViewType;
   ModelWidget *mpModelWidget;
+  bool mVisualizationView;
   QRectF mExtentRectangle;
   bool mIsCustomScale;
   bool mAddClassAnnotationNeeded;
@@ -100,7 +103,6 @@ private:
   Component *mpClickedState;
   bool mIsMovingComponentsAndShapes;
   bool mRenderingLibraryPixmap;
-  bool mSkipFoucusOutEvent;
   QList<Component*> mComponentsList;
   QList<LineAnnotation*> mConnectionsList;
   QList<LineAnnotation*> mTransitionsList;
@@ -126,6 +128,9 @@ private:
   QAction *mpBringForwardAction;
   QAction *mpSendToBackAction;
   QAction *mpSendBackwardAction;
+  QAction *mpCutAction;
+  QAction *mpCopyAction;
+  QAction *mpPasteAction;
   QAction *mpDuplicateAction;
   QAction *mpRotateClockwiseAction;
   QAction *mpRotateAntiClockwiseAction;
@@ -133,34 +138,38 @@ private:
   QAction *mpFlipVerticalAction;
   QAction *mpSetInitialStateAction;
   QAction *mpCancelTransitionAction;
+  // scene->items().contains(...) involves sorting on each items() call, avoid it
+  QSet<QGraphicsItem*> mAllItems;
 public:
-  GraphicsView(StringHandler::ViewType viewType, ModelWidget *parent);
+  GraphicsView(StringHandler::ViewType viewType, ModelWidget *pModelWidget, bool visualizationView = false);
   CoOrdinateSystem mCoOrdinateSystem;
   bool mSkipBackground; /* Do not draw the background rectangle */
   StringHandler::ViewType getViewType() {return mViewType;}
   ModelWidget* getModelWidget() {return mpModelWidget;}
+  bool isVisualizationView() {return mVisualizationView;}
   void setExtentRectangle(qreal x1, qreal y1, qreal x2, qreal y2);
   QRectF getExtentRectangle() {return mExtentRectangle;}
   void setIsCustomScale(bool enable) {mIsCustomScale = enable;}
   bool isCustomScale() {return mIsCustomScale;}
   void setAddClassAnnotationNeeded(bool needed) {mAddClassAnnotationNeeded = needed;}
   bool isAddClassAnnotationNeeded() {return mAddClassAnnotationNeeded;}
-  void setIsCreatingConnection(bool enable);
+  void setIsCreatingConnection(const bool enable);
   bool isCreatingConnection() {return mIsCreatingConnection;}
-  void setIsCreatingTransition(bool enable);
+  void setIsCreatingTransition(const bool enable);
   bool isCreatingTransition() {return mIsCreatingTransition;}
-  void setIsCreatingLineShape(bool enable);
+  void setIsCreatingLineShape(const bool enable);
   bool isCreatingLineShape() {return mIsCreatingLineShape;}
-  void setIsCreatingPolygonShape(bool enable);
+  void setIsCreatingPolygonShape(const bool enable);
   bool isCreatingPolygonShape() {return mIsCreatingPolygonShape;}
-  void setIsCreatingRectangleShape(bool enable);
+  void setIsCreatingRectangleShape(const bool enable);
   bool isCreatingRectangleShape() {return mIsCreatingRectangleShape;}
-  void setIsCreatingEllipseShape(bool enable);
+  void setIsCreatingEllipseShape(const bool enable);
   bool isCreatingEllipseShape() {return mIsCreatingEllipseShape;}
-  void setIsCreatingTextShape(bool enable);
+  void setIsCreatingTextShape(const bool enable);
   bool isCreatingTextShape() {return mIsCreatingTextShape;}
-  void setIsCreatingBitmapShape(bool enable);
+  void setIsCreatingBitmapShape(const bool enable);
   bool isCreatingBitmapShape() {return mIsCreatingBitmapShape;}
+  void setIsCreatingPrologue(const bool enable);
   void setIsPanning(bool enable);
   bool isPanning() {return mIsPanning;}
   void setDragModeInternal(bool enable, bool updateCursor = false);
@@ -174,6 +183,9 @@ public:
   QList<ShapeAnnotation*> getInheritedShapesList() {return mInheritedShapesList;}
   QAction* getManhattanizeAction() {return mpManhattanizeAction;}
   QAction* getDeleteAction() {return mpDeleteAction;}
+  QAction* getCutAction() {return mpCutAction;}
+  QAction* getCopyAction() {return mpCopyAction;}
+  QAction* getPasteAction() {return mpPasteAction;}
   QAction* getDuplicateAction() {return mpDuplicateAction;}
   QAction* getBringToFrontAction() {return mpBringToFrontAction;}
   QAction* getBringForwardAction() {return mpBringForwardAction;}
@@ -185,7 +197,7 @@ public:
   QAction* getFlipVerticalAction() {return mpFlipVerticalAction;}
   bool addComponent(QString className, QPointF position);
   void addComponentToView(QString name, LibraryTreeItem *pLibraryTreeItem, QString annotation, QPointF position,
-                          ComponentInfo *pComponentInfo, bool addObject = true, bool openingClass = false);
+                          ComponentInfo *pComponentInfo, bool addObject, bool openingClass, bool emitComponentAdded);
   void addComponentToList(Component *pComponent) {mComponentsList.append(pComponent);}
   void addInheritedComponentToList(Component *pComponent) {mInheritedComponentsList.append(pComponent);}
   void addComponentToClass(Component *pComponent);
@@ -194,29 +206,33 @@ public:
   void deleteComponentFromList(Component *pComponent) {mComponentsList.removeOne(pComponent);}
   void deleteInheritedComponentFromList(Component *pComponent) {mInheritedComponentsList.removeOne(pComponent);}
   Component* getComponentObject(QString componentName);
-  QString getUniqueComponentName(QString componentName, int number = 1);
+  QString getUniqueComponentName(QString componentName, int number = 0);
   bool checkComponentName(QString componentName);
   QList<Component*> getComponentsList() {return mComponentsList;}
   QList<Component*> getInheritedComponentsList() {return mInheritedComponentsList;}
   QList<LineAnnotation*> getConnectionsList() {return mConnectionsList;}
   QList<LineAnnotation*> getInheritedConnectionsList() {return mInheritedConnectionsList;}
-  void addConnectionToClass(LineAnnotation *pConnectionLineAnnotation);
+  bool addConnectionToClass(LineAnnotation *pConnectionLineAnnotation, bool deleteUndo = false);
   void deleteConnectionFromClass(LineAnnotation *pConnectionLineAnnotation);
   void updateConnectionInClass(LineAnnotation *pConnectionLineAnnotation);
   void addConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.append(pConnectionLineAnnotation);}
   void addInheritedConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.append(pConnectionLineAnnotation);}
   void deleteConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.removeOne(pConnectionLineAnnotation);}
+  void removeConnectionsFromView();
   void deleteInheritedConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.removeOne(pConnectionLineAnnotation);}
+  int numberOfComponentConnections(Component *pComponent, LineAnnotation *pExcludeConnectionLineAnnotation = 0);
   QList<LineAnnotation*> getTransitionsList() {return mTransitionsList;}
   void addTransitionToClass(LineAnnotation *pTransitionLineAnnotation);
   void deleteTransitionFromClass(LineAnnotation *pTransitionLineAnnotation);
   void addTransitionToList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.append(pTransitionLineAnnotation);}
   void deleteTransitionFromList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.removeOne(pTransitionLineAnnotation);}
+  void removeTransitionsFromView();
   QList<LineAnnotation*> getInitialStatesList() {return mInitialStatesList;}
   void addInitialStateToClass(LineAnnotation *pInitialStateLineAnnotation);
   void deleteInitialStateFromClass(LineAnnotation *pInitialStateLineAnnotation);
   void addInitialStateToList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.append(pInitialStateLineAnnotation);}
   void deleteInitialStateFromList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.removeOne(pInitialStateLineAnnotation);}
+  void removeInitialStatesFromView();
   void addShapeToList(ShapeAnnotation *pShape, int index = -1);
   void addInheritedShapeToList(ShapeAnnotation *pShape) {mInheritedShapesList.append(pShape);}
   void deleteShape(ShapeAnnotation *pShapeAnnotation);
@@ -227,9 +243,16 @@ public:
   void bringForward(ShapeAnnotation *pShape);
   void sendToBack(ShapeAnnotation *pShape);
   void sendBackward(ShapeAnnotation *pShape);
+  void clearGraphicsView();
+  void removeClassComponents();
+  void removeInheritedClassShapes();
+  void removeInheritedClassComponents();
+  void removeInheritedClassConnections();
   void removeAllComponents() {mComponentsList.clear();}
   void removeAllShapes() {mShapesList.clear();}
   void removeAllConnections() {mConnectionsList.clear();}
+  void removeAllTransitions() {mTransitionsList.clear();}
+  void removeAllInitialStates() {mInitialStatesList.clear();}
   void createLineShape(QPointF point);
   void createPolygonShape(QPointF point);
   void createRectangleShape(QPointF point);
@@ -238,7 +261,7 @@ public:
   void createBitmapShape(QPointF point);
   QRectF itemsBoundingRect();
   QPointF snapPointToGrid(QPointF point);
-  QPointF movePointByGrid(QPointF point);
+  QPointF movePointByGrid(QPointF point, QPointF origin = QPointF(0, 0), bool useShiftModifier = false);
   QPointF roundPoint(QPointF point);
   bool hasAnnotation();
   void addItem(QGraphicsItem *pGraphicsItem);
@@ -248,17 +271,46 @@ private:
   void createActions();
   bool isClassDroppedOnItself(LibraryTreeItem *pLibraryTreeItem);
   bool isAnyItemSelectedAndEditable(int key);
+  bool isCreatingShape();
+  Component* getComponentFromQGraphicsItem(QGraphicsItem *pGraphicsItem);
+  Component* componentAtPosition(QPoint position);
   Component* connectorComponentAtPosition(QPoint position);
   Component* stateComponentAtPosition(QPoint position);
+  static bool updateComponentConnectorSizingParameter(GraphicsView *pGraphicsView, QString className, Component *pComponent);
+  bool handleDoubleClickOnComponent(QMouseEvent *event);
+  void uncheckAllShapeDrawingActions();
+  void setOriginAdjustAndInitialize(ShapeAnnotation* shapeAnnotation);
+  void setOriginAdjustAndInitialize(PolygonAnnotation* shapeAnnotation);
+  void adjustInitializeDraw(ShapeAnnotation* shapeAnnotation);
+  void finishDrawingGenericShape();
+  void finishDrawingLineShape(bool removeLastAddedPoint = false);
+  void finishDrawingPolygonShape(bool removeLastAddedPoint = false);
+  void finishDrawingRectangleShape();
+  void finishDrawingEllipseShape();
+  void finishDrawingTextShape();
+  void finishDrawingBitmapShape();
+  void checkEmitUpdateSelect(const bool showPropertiesAndSelect, ShapeAnnotation* shapeAnnotation);
+  void copyItems(bool cut);
+  void modelicaGraphicsViewContextMenu(QMenu *pMenu);
+  void modelicaOneShapeContextMenu(ShapeAnnotation *pShapeAnnotation, QMenu *pMenu);
+  void modelicaOneComponentContextMenu(Component *pComponent, QMenu *pMenu);
+  void modelicaMultipleItemsContextMenu(QMenu *pMenu);
+  void compositeModelGraphicsViewContextMenu(QMenu *pMenu);
+  void compositeModelOneShapeContextMenu(ShapeAnnotation *pShapeAnnotation, QMenu *pMenu);
+  void compositeModelOneComponentContextMenu(Component *pComponent, QMenu *pMenu);
+  void compositeModelMultipleItemsContextMenu(QMenu *pMenu);
+  void omsGraphicsViewContextMenu(QMenu *pMenu);
+  void omsOneShapeContextMenu(ShapeAnnotation *pShapeAnnotation, QMenu *pMenu);
+  void omsOneComponentContextMenu(Component *pComponent, QMenu *pMenu);
+  void omsMultipleItemsContextMenu(QMenu *pMenu);
 signals:
-  void mouseManhattanize();
-  void mouseDelete();
+  void manhattanize();
+  void deleteSignal();
   void mouseDuplicate();
   void mouseRotateClockwise();
   void mouseRotateAntiClockwise();
   void mouseFlipHorizontal();
   void mouseFlipVertical();
-  void keyPressDelete();
   void keyPressRotateClockwise();
   void keyPressRotateAntiClockwise();
   void keyPressFlipHorizontal();
@@ -288,7 +340,10 @@ public slots:
   void zoomIn();
   void zoomOut();
   void selectAll();
-  void clearSelection();
+  void cutItems();
+  void copyItems();
+  void pasteItems();
+  void clearSelection(QGraphicsItem *pSelectGraphicsItem = 0);
   void addClassAnnotation(bool alwaysAdd = true);
   void showGraphicsViewProperties();
   void showRenameDialog();
@@ -303,20 +358,20 @@ public slots:
   void setInitialState();
   void cancelTransition();
 protected:
-  virtual void dragMoveEvent(QDragMoveEvent *event);
-  virtual void dropEvent(QDropEvent *event);
-  virtual void drawBackground(QPainter *painter, const QRectF &rect);
-  virtual void mousePressEvent(QMouseEvent *event);
-  virtual void mouseMoveEvent(QMouseEvent *event);
-  virtual void mouseReleaseEvent(QMouseEvent *event);
-  virtual void mouseDoubleClickEvent(QMouseEvent *event);
-  virtual void focusOutEvent(QFocusEvent *event);
-  virtual void keyPressEvent(QKeyEvent *event);
-  virtual void keyReleaseEvent(QKeyEvent *event);
-  virtual void contextMenuEvent(QContextMenuEvent *event);
-  virtual void resizeEvent(QResizeEvent *event);
-  virtual void wheelEvent(QWheelEvent *event);
-  virtual void leaveEvent(QEvent *event);
+  virtual void dragMoveEvent(QDragMoveEvent *event) override;
+  virtual void dropEvent(QDropEvent *event) override;
+  virtual void drawBackground(QPainter *painter, const QRectF &rect) override;
+  virtual void mousePressEvent(QMouseEvent *event) override;
+  virtual void mouseMoveEvent(QMouseEvent *event) override;
+  virtual void mouseReleaseEvent(QMouseEvent *event) override;
+  virtual void mouseDoubleClickEvent(QMouseEvent *event) override;
+  virtual void focusOutEvent(QFocusEvent *event) override;
+  virtual void keyPressEvent(QKeyEvent *event) override;
+  virtual void keyReleaseEvent(QKeyEvent *event) override;
+  virtual void contextMenuEvent(QContextMenuEvent *event) override;
+  virtual void resizeEvent(QResizeEvent *event) override;
+  virtual void wheelEvent(QWheelEvent *event) override;
+  virtual void leaveEvent(QEvent *event) override;
 };
 
 class WelcomePageWidget : public QWidget
@@ -356,6 +411,62 @@ private slots:
   void openLatestNewsItem(QListWidgetItem *pItem);
 };
 
+class UndoCommand;
+class UndoStack : public QUndoStack
+{
+  Q_OBJECT
+public:
+  UndoStack(QObject *parent = 0);
+  void push(UndoCommand *cmd);
+
+  bool isEnabled() {return mEnabled;}
+  void setEnabled(bool enable) {mEnabled = enable;}
+private:
+  bool mEnabled;
+};
+
+class IconDiagramMap
+{
+public:
+  IconDiagramMap()
+  {
+    mExtent.clear();
+    mExtent << QPointF(0, 0) << QPointF(0, 0);
+    mPrimitivesVisible = true;
+  }
+
+  QList<QPointF> mExtent;
+  bool mPrimitivesVisible;
+};
+
+class MimeData : public QMimeData
+{
+  Q_OBJECT
+public:
+  MimeData() : QMimeData()
+  {
+    mComponents.clear();
+    mConnections.clear();
+    mShapes.clear();
+  }
+  void addComponent(Component *pComponent) {mComponents.append(pComponent);}
+  QList<Component*> getComponents() const {return mComponents;}
+  void addConnection(LineAnnotation *pConnectionLineAnnotation) {mConnections.append(pConnectionLineAnnotation);}
+  QList<LineAnnotation*> getConnections() const {return mConnections;}
+  void addShape(ShapeAnnotation *pShapeAnnotation) {mShapes.append(pShapeAnnotation);}
+  QList<ShapeAnnotation*> getShapes() const {return mShapes;}
+private:
+  QList<Component*> mComponents;
+  QList<LineAnnotation*> mConnections;
+  QList<ShapeAnnotation*> mShapes;
+  // QMimeData interface
+public:
+  virtual QStringList formats() const override
+  {
+    return QStringList() << "text/plain" << Helper::cutCopyPasteFormat;
+  }
+};
+
 class ModelWidgetContainer;
 class ModelicaHighlighter;
 class CompositeModelHighlighter;
@@ -374,7 +485,7 @@ public:
   QToolButton* getDocumentationViewToolButton() {return mpDocumentationViewToolButton;}
   GraphicsView* getDiagramGraphicsView() {return mpDiagramGraphicsView;}
   GraphicsView* getIconGraphicsView() {return mpIconGraphicsView;}
-  QUndoStack* getUndoStack() {return mpUndoStack;}
+  UndoStack* getUndoStack() {return mpUndoStack;}
   BaseEditor* getEditor() {return mpEditor;}
   void setModelClassPathLabel(QString path) {mpModelClassPathLabel->setText(path);}
   void setModelFilePathLabel(QString path) {mpModelFilePathLabel->setText(path);}
@@ -383,35 +494,46 @@ public:
   void removeInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.removeOne(pLibraryTreeItem);}
   void clearInheritedClasses() {mInheritedClassesList.clear();}
   QList<LibraryTreeItem*> getInheritedClassesList() {return mInheritedClassesList;}
+  QMap<int, IconDiagramMap> getInheritedClassIconMap() {return mInheritedClassesIconMap;}
+  QMap<int, IconDiagramMap> getInheritedClassDiagramMap() {return mInheritedClassesDiagramMap;}
+  const QList<ComponentInfo*> &getComponentsList() {return mComponentsList;}
   QMap<QString, QString> getExtendsModifiersMap(QString extendsClass);
+  QMap<QString, QString> getDerivedClassModifiersMap();
   void fetchExtendsModifiers(QString extendsClass);
   void reDrawModelWidgetInheritedClasses();
   void drawBaseCoOrdinateSystem(ModelWidget *pModelWidget, GraphicsView *pGraphicsView);
+  void drawModelIconDiagramShapes(QStringList shapes, GraphicsView *pGraphicsView, bool select);
   ShapeAnnotation* createNonExistingInheritedShape(GraphicsView *pGraphicsView);
-  ShapeAnnotation* createInheritedShape(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView);
+  static ShapeAnnotation* createInheritedShape(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView);
   Component* createInheritedComponent(Component *pComponent, GraphicsView *pGraphicsView);
   LineAnnotation* createInheritedConnection(LineAnnotation *pConnectionLineAnnotation);
   void loadComponents();
   void loadDiagramView();
   void loadConnections();
+  void getModelConnections();
+  void addConnection(QStringList connectionList, QString connectionAnnotationString, bool addToOMC, bool select);
   void createModelWidgetComponents();
+  ShapeAnnotation* drawOMSModelElement();
   Component* getConnectorComponent(Component *pConnectorComponent, QString connectorName);
   void clearGraphicsViews();
   void reDrawModelWidget();
   bool validateText(LibraryTreeItem **pLibraryTreeItem);
   bool modelicaEditorTextChanged(LibraryTreeItem **pLibraryTreeItem);
   void updateChildClasses(LibraryTreeItem *pLibraryTreeItem);
+  bool omsimulatorEditorTextChanged();
   void clearSelection();
   void updateClassAnnotationIfNeeded();
-  void updateModelText(bool updateText = true);
+  void updateModelText();
   void updateModelicaTextManually(QString contents);
   void updateUndoRedoActions();
-  void updateDynamicResults(QString resultFileName);
-  QString getResultFileName() {return mResultFileName;}
   bool writeCoSimulationResultFile(QString fileName);
   bool writeVisualXMLFile(QString fileName, bool canWriteVisualXMLFile = false);
   void beginMacro(const QString &text);
   void endMacro();
+  void updateViewButtonsBasedOnAccess();
+  void associateBusWithConnector(QString busName, QString connectorName);
+  void dissociateBusWithConnector(QString busName, QString connectorName);
+  void associateBusWithConnectors(QString busName);
 private:
   ModelWidgetContainer *mpModelWidgetContainer;
   LibraryTreeItem *mpLibraryTreeItem;
@@ -430,7 +552,7 @@ private:
   GraphicsScene *mpDiagramGraphicsScene;
   GraphicsView *mpIconGraphicsView;
   GraphicsScene *mpIconGraphicsScene;
-  QUndoStack *mpUndoStack;
+  UndoStack *mpUndoStack;
   QUndoView *mpUndoView;
   BaseEditor *mpEditor;
   QStatusBar *mpModelStatusBar;
@@ -440,24 +562,23 @@ private:
   bool mCreateModelWidgetComponents;
   bool mExtendsModifiersLoaded;
   QMap<QString, QMap<QString, QString> > mExtendsModifiersMap;
+  bool mDerivedClassModifiersLoaded;
+  QMap<QString, QString> mDerivedClassModifiersMap;
   QList<LibraryTreeItem*> mInheritedClassesList;
+  QMap<int, IconDiagramMap> mInheritedClassesIconMap;
+  QMap<int, IconDiagramMap> mInheritedClassesDiagramMap;
   QList<ComponentInfo*> mComponentsList;
   QStringList mComponentsAnnotationsList;
-  QString mResultFileName;
 
+  IconDiagramMap getIconDiagramMap(QString mapAnnotation);
   void getModelInheritedClasses();
   void drawModelInheritedClassShapes(ModelWidget *pModelWidget, StringHandler::ViewType viewType);
-  void removeInheritedClassShapes(StringHandler::ViewType viewType);
   void getModelIconDiagramShapes(StringHandler::ViewType viewType);
   void drawModelInheritedClassComponents(ModelWidget *pModelWidget, StringHandler::ViewType viewType);
-  void removeInheritedClassComponents(StringHandler::ViewType viewType);
-  void removeClassComponents(StringHandler::ViewType viewType);
   void getModelComponents();
   void drawModelIconComponents();
   void drawModelDiagramComponents();
   void drawModelInheritedClassConnections(ModelWidget *pModelWidget);
-  void removeInheritedClassConnections();
-  void getModelConnections();
   void getModelTransitions();
   void getModelInitialStates();
   void getMetaModelSubModels();
@@ -466,6 +587,12 @@ private:
   QString getCompositeModelName();
   void getCompositeModelSubModels();
   void getCompositeModelConnections();
+  void drawOMSModelIconElements();
+  void drawOMSModelDiagramElements();
+  void drawOMSModelConnections();
+  void associateBusWithConnector(QString busName, QString connectorName, GraphicsView *pGraphicsView);
+  void dissociateBusWithConnector(QString busName, QString connectorName, GraphicsView *pGraphicsView);
+  void associateBusWithConnectors(Component *pBusComponent, GraphicsView *pGraphicsView);
 private slots:
   void showIconView(bool checked);
   void showDiagramView(bool checked);
@@ -476,12 +603,10 @@ public slots:
   bool compositeModelEditorTextChanged();
   void handleCanUndoChanged(bool canUndo);
   void handleCanRedoChanged(bool canRedo);
-  void removeDynamicResults(QString resultFileName = "");
 protected:
-  virtual void closeEvent(QCloseEvent *event);
+  virtual void closeEvent(QCloseEvent *event) override;
 };
 
-
 void addCloseActionsToSubWindowSystemMenu(QMdiSubWindow *pMdiSubWindow);
 
 class ModelWidgetContainer : public QMdiArea
@@ -503,11 +628,13 @@ public:
 #if !defined(WITHOUT_OSG)
   void updateThreeDViewer(ModelWidget *pModelWidget);
 #endif
+  bool validateText();
 private:
   StringHandler::ViewType mPreviousViewType;
   bool mShowGridLines;
   QDialog *mpModelSwitcherDialog;
   QListWidget *mpRecentModelsList;
+  QMdiSubWindow *mpLastActiveSubWindow;
   void loadPreviousViewType(ModelWidget *pModelWidget);
 public slots:
   bool openRecentModelWidget(QListWidgetItem *pListWidgetItem);
@@ -519,6 +646,13 @@ public slots:
   void printModel();
   void showSimulationParams();
   void alignInterfaces();
+  void addSystem();
+  void addOrEditIcon();
+  void deleteIcon();
+  void addConnector();
+  void addBus();
+  void addTLMBus();
+  void addSubModel();
 };
 
 #endif // MODELWIDGETCONTAINER_H
